import { PaymentMethod } from '@bigcommerce/checkout-sdk';
import { withFormik, FormikProps, WithFormikConfig } from 'formik';
import { isNil, noop, omitBy } from 'lodash';
import React, { memo, useCallback, useContext, useMemo, FunctionComponent } from 'react';
import { ObjectSchema } from 'yup';

import { withLanguage, WithLanguageProps } from '../locale';
import { TermsConditions } from '../termsConditions';
import { Fieldset, Form, FormContext } from '../ui/form';

import { DocumentOnlyCustomFormFieldsetValues, SepaCustomFormFieldsetValues } from './checkoutcomFieldsets';
import { CreditCardFieldsetValues } from './creditCard';
import getPaymentValidationSchema from './getPaymentValidationSchema';
import { HostedCreditCardFieldsetValues } from './hostedCreditCard';
import { getUniquePaymentMethodId, PaymentMethodId, PaymentMethodList } from './paymentMethod';
import { CardInstrumentFieldsetValues } from './storedInstrument';
import { StoreCreditField, StoreCreditOverlay } from './storeCredit';
import PaymentRedeemables from './PaymentRedeemables';
import PaymentSubmitButton from './PaymentSubmitButton';
import SpamProtectionField from './SpamProtectionField';
import PaymentSubmitButtonone from './PaymentSubmitButtonone';
import $ from 'jquery';

export interface PaymentFormProps {
    availableStoreCredit?: number;
    defaultGatewayId?: string;
    defaultMethodId: string;
    didExceedSpamLimit?: boolean;
    isEmbedded?: boolean;
    isInitializingPayment?: boolean;
    isTermsConditionsRequired?: boolean;
    isUsingMultiShipping?: boolean;
    isStoreCreditApplied: boolean;
    methods: PaymentMethod[];
    selectedMethod?: PaymentMethod;
    shouldShowStoreCredit?: boolean;
    shouldDisableSubmit?: boolean;
    shouldHidePaymentSubmitButton?: boolean;
    shouldExecuteSpamCheck?: boolean;
    termsConditionsText?: string;
    termsConditionsUrl?: string;
    usableStoreCredit?: number;
    validationSchema?: ObjectSchema<Partial<PaymentFormValues>>;
    isPaymentDataRequired(): boolean;
    onMethodSelect?(method: PaymentMethod): void;
    onStoreCreditChange?(useStoreCredit?: boolean): void;
    onSubmit?(values: PaymentFormValues): void;
    onUnhandledError?(error: Error): void;
}

export type PaymentFormValues = (
    CreditCardFieldsetValues & PaymentFormCommonValues |
    CardInstrumentFieldsetValues & PaymentFormCommonValues |
    HostedCreditCardFieldsetValues & PaymentFormCommonValues |
    HostedWidgetPaymentMethodValues & PaymentFormCommonValues |
    DocumentOnlyCustomFormFieldsetValues & PaymentFormCommonValues |
    SepaCustomFormFieldsetValues & PaymentFormCommonValues |
    PaymentFormCommonValues
);

export interface PaymentFormCommonValues {
    paymentProviderRadio: string; // TODO: Give this property a better name. We need to keep it for now because of legacy reasons.
    terms?: boolean;
}

export interface HostedWidgetPaymentMethodValues {
    shouldSaveInstrument: boolean;
}

const PaymentForm: FunctionComponent<PaymentFormProps & FormikProps<PaymentFormValues> & WithLanguageProps> = ({
    availableStoreCredit = 0,
    didExceedSpamLimit,
    isEmbedded,
    isInitializingPayment,
    isPaymentDataRequired,
    isTermsConditionsRequired,
    isStoreCreditApplied,
    isUsingMultiShipping,
    methods,
    onMethodSelect,
    onStoreCreditChange,
    onUnhandledError,
    resetForm,
    selectedMethod,
    shouldDisableSubmit,
    shouldHidePaymentSubmitButton,
    shouldExecuteSpamCheck,
    termsConditionsText = '',
    termsConditionsUrl,
    usableStoreCredit = 0,
    values,
}) => {
    const selectedMethodId = useMemo(() => {
        if (!selectedMethod) {
            return;
        }

        switch (selectedMethod.id) {
        case PaymentMethodId.AmazonPay:
            if (selectedMethod.initializationData.paymentToken) {
                return;
            }

            return selectedMethod.id;

        default:
            return selectedMethod.id;
        }
    }, [selectedMethod]);

    //0420 payment  onclick  even
    const paymentClick = useCallback(() => {
        console.log("paymentClick");
        var firstname="",familyname="",customerEmail="";

        if(document.getElementsByClassName("first-name").length>0){
             firstname =document.getElementsByClassName("first-name")[0].textContent as string; 
             familyname=document.getElementsByClassName("family-name")[0].textContent as string; 
        }
        if(document.querySelector(".checkout-step--customer .customerView-body[data-test='customer-info']")!=undefined){
            customerEmail=document.querySelector(".checkout-step--customer .customerView-body[data-test='customer-info']")?.textContent as  string;
        }
        var file=document.querySelector("#customerfile") as HTMLInputElement;
        var customFiles = file.files || [] ;
        if (customFiles.length > 0) {


            var action = 'https://getform.io/f/650792a9-5512-417e-b917-f1ca1bb6a962';

            
            if ((window.location.hostname.includes("neteonnet-store-1.mybigcommerce.com")) || 
                (window.location.hostname.includes("store-uraahlkkmm.mybigcommerce.com")) || 
                (window.location.hostname.includes("shopmoxa.neteon.net")) )
            {
                //moxa
                action = 'https://getform.io/f/650792a9-5512-417e-b917-f1ca1bb6a962';
            } 
            else if ((window.location.hostname.includes("neteonnet-store-2.mybigcommerce.com")) || 
                (window.location.hostname.includes("store-exhcjslbho.mybigcommerce.com")) || 
                (window.location.hostname.includes("neteon.net") && (!window.location.hostname.includes("shopmoxa.neteon.net"))) ) 
            {
                //Neteon
                action = 'https://getform.io/f/2d59d05d-0b46-4a36-9a51-71ad0e9aa109';
            }


            var data=new FormData();
            data.append("customer-name",firstname+familyname);
            data.append("customer-email",customerEmail);
            data.append("customer-file", customFiles[0]);
            $.ajax({
                type: 'POST',
                url: action,
                crossDomain: true,
                data: data,
                dataType: 'json',
                contentType: false,
                processData: false,
                headers: {
                    "Accept": "application/json"
                }
            }).done(function(){
                document.getElementById("checkout-payment-continue")?.click();

            }).fail(function(error){
                alert('email  error');
                console.log(error.responseText);
            })
        } else {
            document.getElementById("checkout-payment-continue")?.click();
        }

    }, []);

    if (shouldExecuteSpamCheck) {
        return <SpamProtectionField
            didExceedSpamLimit={ didExceedSpamLimit }
            onUnhandledError={ onUnhandledError }
        />;
    }

    return (
        <Form
            className="checkout-form"
            testId="payment-form"
        >
            { usableStoreCredit > 0 && <StoreCreditField
                availableStoreCredit={ availableStoreCredit }
                isStoreCreditApplied={ isStoreCreditApplied }
                name="useStoreCredit"
                onChange={ onStoreCreditChange }
                usableStoreCredit={ usableStoreCredit }
            /> }

            <PaymentMethodListFieldset
                isEmbedded={ isEmbedded }
                isInitializingPayment={ isInitializingPayment }
                isPaymentDataRequired={ isPaymentDataRequired }
                isUsingMultiShipping={ isUsingMultiShipping }
                methods={ methods }
                onMethodSelect={ onMethodSelect }
                onUnhandledError={ onUnhandledError }
                resetForm={ resetForm }
                values={ values }
            />

            <PaymentRedeemables />

            { isTermsConditionsRequired && <TermsConditions
                termsConditionsText={ termsConditionsText }
                termsConditionsUrl={ termsConditionsUrl }
            /> }

            <div className="form-actions">
            { shouldHidePaymentSubmitButton ?
                    <PaymentMethodSubmitButtonContainer /> :
                    <PaymentSubmitButtonone
                        isDisabled={ shouldDisableSubmit }
                        methodGateway={ selectedMethod && selectedMethod.gateway }
                        methodId={ selectedMethodId }
                        methodType={ selectedMethod && selectedMethod.method }
                        paymentClick={paymentClick}
                    /> }

                { shouldHidePaymentSubmitButton ?
                    <PaymentMethodSubmitButtonContainer /> :
                    <PaymentSubmitButton
                        isDisabled={ shouldDisableSubmit }
                        methodGateway={ selectedMethod && selectedMethod.gateway }
                        methodId={ selectedMethodId }
                        methodType={ selectedMethod && selectedMethod.method }
                        classname="paymentsub"
                    /> }
            </div>
        </Form>
    );
};

const PaymentMethodSubmitButtonContainer: FunctionComponent = () => {
    return <div className="submitButtonContainer" id="checkout-payment-continue" />;
};

interface PaymentMethodListFieldsetProps {
    isEmbedded?: boolean;
    isInitializingPayment?: boolean;
    isUsingMultiShipping?: boolean;
    methods: PaymentMethod[];
    values: PaymentFormValues;
    isPaymentDataRequired(): boolean;
    onMethodSelect?(method: PaymentMethod): void;
    onUnhandledError?(error: Error): void;
    resetForm(nextValues?: PaymentFormValues): void;
}

const PaymentMethodListFieldset: FunctionComponent<PaymentMethodListFieldsetProps> = ({
    isEmbedded,
    isInitializingPayment,
    isPaymentDataRequired,
    isUsingMultiShipping,
    methods,
    onMethodSelect = noop,
    onUnhandledError,
    resetForm,
    values,
}) => {
    const { setSubmitted } = useContext(FormContext);

    const commonValues = useMemo(
        () => ({ terms: values.terms }),
        [values.terms]
    );

    const handlePaymentMethodSelect = useCallback((method: PaymentMethod) => {
        resetForm({
            ...commonValues,
            ccCustomerCode: '',
            ccCvv: '',
            ccDocument: '',
            ccExpiry: '',
            ccName: '',
            ccNumber: '',
            instrumentId: '',
            paymentProviderRadio: getUniquePaymentMethodId(method.id, method.gateway),
            shouldSaveInstrument: false,
        });

        setSubmitted(false);
        onMethodSelect(method);
    }, [
        commonValues,
        onMethodSelect,
        resetForm,
        setSubmitted,
    ]);

    return (
        <Fieldset>
            { !isPaymentDataRequired() && <StoreCreditOverlay /> }

            <PaymentMethodList
                isEmbedded={ isEmbedded }
                isInitializingPayment={ isInitializingPayment }
                isUsingMultiShipping={ isUsingMultiShipping }
                methods={ methods }
                onSelect={ handlePaymentMethodSelect }
                onUnhandledError={ onUnhandledError }
            />
        </Fieldset>
    );
};

const paymentFormConfig: WithFormikConfig<PaymentFormProps & WithLanguageProps, PaymentFormValues> = {
    mapPropsToValues: ({
        defaultGatewayId,
        defaultMethodId,

    }) => ({
        ccCustomerCode: '',
        ccCvv: '',
        ccDocument: '',
        ccExpiry: '',
        ccName: '',
        ccNumber: '',
        paymentProviderRadio: getUniquePaymentMethodId(defaultMethodId, defaultGatewayId),
        instrumentId: '',
        shouldSaveInstrument: false,
        terms: false,
        hostedForm: {
            cardType: '',
            errors: {
                cardCode: '',
                cardCodeVerification: '',
                cardExpiry: '',
                cardName: '',
                cardNumber: '',
                cardNumberVerification: '',
            },
        },
    }),

    handleSubmit: (values, { props: { onSubmit = noop } }) => {
        onSubmit(omitBy(values, (value, key) =>
            isNil(value) || value === '' || key === 'hostedForm'
        ));
    },

    validationSchema: ({
        language,
        isTermsConditionsRequired = false,
        validationSchema,
    }: PaymentFormProps & WithLanguageProps) => (
        getPaymentValidationSchema({
            additionalValidation: validationSchema,
            isTermsConditionsRequired,
            language,
        })
    ),
};

export default withLanguage(withFormik(paymentFormConfig)(memo(PaymentForm)));
